@@ -13,53 +13,65 @@
 # You should have received a copy of the GNU General Public License
 # along with this program.  If not, see <http://www.gnu.org/licenses/>.
 
-import converter
 import upmplgmodels
+from subsonic_connector.album import Album
+import album_util
+from msgproc_provider import msgproc
 
 
-def set_track_number(track_number : str, target : dict):
+def set_track_number(track_number: str, target: dict):
     target['upnp:originalTrackNumber'] = track_number
 
 
-def set_album_art_from_uri(album_art_uri : str, target : dict):
-    target['upnp:albumArtURI'] = album_art_uri
+def get_album_art_uri(entry: dict):
+    return entry['upnp:albumArtURI'] if 'upnp:albumArtURI' in entry else None
 
 
-def set_album_title(album_title : str, target : dict):
+def set_album_art_from_uri(album_art_uri: str, target: dict):
+    if album_art_uri:
+        target['upnp:albumArtURI'] = album_art_uri
+
+
+def set_album_title(album_title: str, target: dict):
     target['tt'] = album_title
 
 
-def set_album_id(album_id : str, target : dict):
+def set_album_id(album_id: str, target: dict):
     target['album_id'] = album_id
 
 
-def set_class(upnp_class : str, target : dict):
+def set_class(upnp_class: str, target: dict):
     target['upnp:class'] = upnp_class
 
 
-def set_artist(artist : str, target : dict):
+def set_artist(artist: str, target: dict):
     target['upnp:artist'] = artist
 
 
-def set_class_music_track(target : dict):
+def set_class_music_track(target: dict):
     target['upnp:class'] = upmplgmodels.Track.upnpclass
 
 
-def set_class_album(target : dict):
+def set_class_album(target: dict):
     target['upnp:class'] = upmplgmodels.Album.upnpclass
 
 
-def set_class_artist(target : dict):
+def set_date_from_album(album: Album, target: dict):
+    date_str: str = album_util.getOriginalReleaseDate(album)
+    if not date_str:
+        # fallback to year
+        date_str = str(album.getYear()) if album.getYear() else ""
+    set_date_str(date_str=date_str, target=target)
+
+
+def set_date_str(date_str: str, target: dict):
+    target['dc:date'] = date_str
+
+
+def set_class_artist(target: dict):
     target['upnp:class'] = upmplgmodels.Artist.upnpclass
 
 
-def set_album_art_from_album_id(
-        album_id : str,
-        target : dict[str, any]):
-    art_uri : str = converter.converter_album_id_to_url(album_id) if album_id else None
-    if art_uri: set_album_art_from_uri(art_uri, target)
-
-
 def set_bit_depth(bit_depth: int, target: dict):
     target['res:bitsPerSample'] = str(bit_depth)
 
@@ -74,3 +86,9 @@ def set_sample_rate(sample_rate: int, target: dict):
 
 def set_bit_rate(bit_rate: int, target: dict):
     target['kbs'] = str(bit_rate)
+
+
+def set_metadata(metadata_name: str, metadata_value: str, target: dict):
+    k: str = f"upmpd:{metadata_name}"
+    msgproc.log(f"set_metadata setting [{k}] to [{metadata_value}]")
+    target[k] = metadata_value
